#!/usr/bin/python

" Module to deal with schwinn810 configuration file "

from gi.repository import Gtk
import os, stat, json, logging, sys
from appdirs import AppDirs
from collections import defaultdict
import constants as config
# import gettext, locale
from pkg_resources import resource_string
from i18n import _, bind
from utils import read_settings
import pytz, re

_log = logging.getLogger(__name__)

# _ = i18n.underscore

print(_("Hello"))

class SettingsDialog:
    """Schwinn810 settings dialog used to read/write JSON formatted
configuration.

Configuration is expected at standard location (XDG on GNU/Linux and
%APP_DATA% on Windows).

    """

    def __init__(self):
        self._log = _log.getChild(self.__class__.__name__)
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain(config.APP)
        self.builder.add_from_string(
            resource_string(__name__, "SettingsDialog.glade").decode('utf-8'))
        self.builder.connect_signals(self)

        # iter = self.liststore.append()
        # iter = self.liststore.append([False, "Garmin Connect", "world", "hello"])
        self.filechooserbutton.add_shortcut_folder("/usr/share/schwinn810")
        self.filechooserbutton.set_filename("/usr/share/schwinn810/babelize.sh")
        # self.filechooserbutton.add_shortcut_folder_uri("file:///usr/share/schwinn810/babelize.sh")

        for zone in pytz.all_timezones:
            self.tzlist.append((zone,))

        def cb_match_func(completion, key, it, data):
            "Custom completion match routine"
            return re.search(key, data[it][0], re.IGNORECASE)

        self.entrycompletion.set_match_func(cb_match_func, self.tzlist)

        self.cbPort.set_active(0)
        # self.cbInterface.set_active(0)

        noperm = False
        try:
            import pwd, grp, os
            noperm = pwd.getpwuid(os.getuid()).pw_name not in grp.getgrnam('dialout').gr_mem
        except:
            self._log.debug("Perhaps non-unix")
            pass
        if noperm:
            self.labelPermissions.show()
            self.infobar.show()

    def load(self, settings=None):
        "Set up GUI controls from settings dictionary"
        if not settings:
            self.labelNoConfig.show()
            self.infobar.show()
            return

        self.settings = settings
        self.cbAutostart.set_active(self.settings[config.AUTOSTART])
        self.cbDump.set_active(self.settings[config.DUMP])
        self.cbNotify.set_active(self.settings[config.NOTIFY])
        self.cbPP.set_active(self.settings[config.PP])
        if self.settings[config.PORT]:
            self.cbPort.get_child().set_text(self.settings[config.PORT])
        if self.settings[config.HOOK]:
            self.filechooserbutton.set_filename(self.settings[config.HOOK])
        if self.settings[config.DIR]:
            self.fcbDestination.set_filename(self.settings[config.DIR])
        self.cbDelete.set_active(self.settings[config.DELETE])
        if self.settings[config.SUFFIX]:
            self.entrySuffix.set_text(self.settings[config.SUFFIX])

        # if self.settings[config.INTERFACE]:
        #     self.entryInterface.set_text(self.settings[config.INTERFACE])
        self.tzEntry.set_text(self.settings[config.CSV2TCX][config.TZ])
        self.cbToGMT.set_active(self.settings[config.CSV2TCX][config.TOGMT])

        head = ["enabled", "name", "password"]
        services = dict()
        entry = dict(zip(head, [False, "user", "password"]))
        services["Map My Run"] = entry
        # entry = dict(zip(head, [False, "world", "hello"]))
        services["Garmin Connect"] = entry
        services.update(self.settings["Upload"])
        self.lsUpload.clear()
        for key, val in services.iteritems():
            # vv = map(lambda x: val[x], head)
            vv = [val[x] for x in head]
            self.lsUpload.append([vv[0], key] + vv[1:])

    def _save(self):
        self.settings[config.AUTOSTART] = self.cbAutostart.get_active()
        self.settings[config.DUMP] = self.cbDump.get_active()
        self.settings[config.NOTIFY] = self.cbNotify.get_active()
        self.settings[config.PP] = self.cbPP.get_active()
        self.settings[config.PORT] = self.cbPort.get_child().get_text()
        self.settings[config.HOOK] = self.filechooserbutton.get_filename()
        self.settings[config.DIR] = self.fcbDestination.get_filename()
        self.settings[config.DELETE] = self.cbDelete.get_active()
        self.settings[config.SUFFIX] = self.entrySuffix.get_text()
        self.settings[config.INTERFACE] = 'gtk'

        self.settings[config.CSV2TCX] = {
          config.TZ : self.tzEntry.get_text(),
          config.TOGMT : self.cbToGMT.get_active()
        }

        self.settings[config.TCX2WEB] = dict()
        for i in self.lsUpload:
            entry = dict(zip(["enabled", "service", "name", "password"], i[:]))
            service = entry.pop("service")
            self.settings[config.TCX2WEB][service] = entry

    def __getattr__(self, name):
        obj = self.builder.get_object(name)
        if obj is None:
            raise AttributeError("Can't resolve '{:s}' from Glade".format(name))
        setattr(self, name, obj)
        return obj

    def run(self):
        ret = self.dialogSettings.run()
        self.dialogSettings.destroy()
        return ret

    def on_buttonOk_clicked(self, widget):
        self._save()

    def on_buttonDismiss_clicked(self, widget):
        self.infobar.hide()

    def on_cellUser_edited(self, widget, path, value):
        it = self.lsUpload.get_iter(path)
        self.lsUpload.set_value(it, 2, value)

    def on_cellPassword_edited(self, widget, path, value):
        it = self.lsUpload.get_iter(path)
        self.lsUpload.set_value(it, 3, value)

    def on_enabled_toggled(self, widget, path):
        it = self.lsUpload.get_iter(path)
        val = self.lsUpload.get_value(it, 0)
        self.lsUpload.set_value(it, 0, not val)

# print dirs.user_data_dir
# print dirs.user_cache_dir
# print dirs.user_log_dir

def main(settings=None):
    if settings is None:
        settings = read_settings()
    s = SettingsDialog()
    s.load(settings)
    if s.run() > 0:               # Gtk.ResponseType.OK
        dirs = AppDirs(config.APP, config.VENDOR)
        try:
            os.makedirs(dirs.user_data_dir)
        except OSError:
            pass
        fname = os.path.join(dirs.user_data_dir, config.FILE)
        with open(fname, "w") as f:
            json.dump(s.settings, f, indent=True)
        os.chmod(fname, stat.S_IWRITE | stat.S_IREAD)

if __name__ == '__main__':
    # locale.setlocale(locale.LC_ALL, '')
    bind()
    logging.basicConfig(level=logging.DEBUG)
    main()
