#!/usr/bin/env python

from gi.repository import Gtk, GLib, Notify
import dbus
from time import sleep
from subprocess import Popen
import constants as config
from pkg_resources import resource_string
from SettingsDialog import main as show_settings
import i18n, os
from pyudev import Context, Monitor
from schwinn810 import Device, Writer
from schwinn810.progress_gtk import GtkProgress
from utils import read_settings
from csv2tcx import convert

class DeviceListener:
    "GUI independent udev listener"

    path = None

    def __init__(self, tray):
        self.tray = tray
        self.settings = self.tray.settings
        context = Context()
        self.monitor = Monitor.from_netlink(context)
        self.monitor.filter_by(subsystem='tty')

    def process_udev_event(self, monitor, condition):#, cb, filters):
        if condition == GLib.IO_IN:
            event = monitor.receive_device()
            if event:
                action, device = event
                # print ("***", action, device)
                if action == 'add' and device['ID_VENDOR_ID'] == '10c4' and device['ID_MODEL_ID'] == 'ea61':
                    self.path = device['DEVPATH']
                    self.tray.notify_connect()
                    sleep(2)
                    if self.settings:
                        d = Device(self.settings[config.PORT], self.settings[config.DUMP])
                        w = Writer(self.settings[config.DIR], self.settings[config.SUFFIX], self.settings[config.HOOK])
                        p = GtkProgress()
                        d.read(w, p)
                        if self.settings[config.DELETE]:
                            d.clear()
                        d.close()
                        Notify.init ("Hello world")
                        Hello=Notify.Notification.new("Schwinn 810 GPS watch", "Data extracted","dialog-information")
                        Hello.set_category('transfer.complete')
                        Hello.set_urgency(Notify.Urgency.LOW)
                        Hello.show()
                    # from download import main
                    # main()
                    self.tray.notify_charge()
                    # Popen("schwinn810")
                elif action == 'remove' and device['DEVPATH'] == self.path:
                    self.tray.notify_disconnect()
        return True

    def hook(self, name):
        settings = self.settings[config.CSV2TCX]
        convert(name, tz=settings[config.TZ], nogmt=not settings[config.TOGMT])

# ACTION => remove
# DEVLINKS => /dev/schwinn810 /dev/serial/by-id/usb-Silicon_Labs_CP2102_USB_to_UART_Bridge_Controller_8887-if00-port0 /dev/serial/by-path/pci-0000:00:1d.1-usb-0:1:1.0-port0
# DEVNAME => /dev/ttyUSB0
# DEVPATH => /devices/pci0000:00/0000:00:1d.1/usb3/3-1/3-1:1.0/ttyUSB0/tty/ttyUSB0
# ID_BUS => usb
# ID_MM_CANDIDATE => 1
# ID_MODEL => CP2102_USB_to_UART_Bridge_Controller
# ID_MODEL_ENC => CP2102\x20USB\x20to\x20UART\x20Bridge\x20Controller
# ID_MODEL_FROM_DATABASE => CP210x UART Bridge
# ID_MODEL_ID => ea61
# ID_PATH => pci-0000:00:1d.1-usb-0:1:1.0
# ID_PATH_TAG => pci-0000_00_1d_1-usb-0_1_1_0
# ID_REVISION => 0100
# ID_SERIAL => Silicon_Labs_CP2102_USB_to_UART_Bridge_Controller_8887
# ID_SERIAL_SHORT => 8887
# ID_TYPE => generic
# ID_USB_DRIVER => cp210x
# ID_USB_INTERFACES => :ff0000:
# ID_USB_INTERFACE_NUM => 00
# ID_VENDOR => Silicon_Labs
# ID_VENDOR_ENC => Silicon\x20Labs
# ID_VENDOR_FROM_DATABASE => Cygnal Integrated Products, Inc.
# ID_VENDOR_ID => 10c4
# MAJOR => 188
# MINOR => 0
# SEQNUM => 2049
# SUBSYSTEM => tty
# UDEV_LOG => 3
# USEC_INITIALIZED => 91648581

class Schwinn810Tray:
    "System tray class"

    def __init__(self):
        self.noshow = False     # glitch with modal dialog & status icon
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain(config.APP)
        self.builder.add_from_string(resource_string(__name__, "tray_gtk.glade").decode('utf-8'))
        self.builder.connect_signals(self)

        self.settings = read_settings()

        self.statusicon.set_from_stock(Gtk.STOCK_DISCONNECT)

        dl = DeviceListener(self)
        GLib.io_add_watch(dl.monitor, GLib.PRIORITY_LOW, GLib.IO_IN, dl.process_udev_event)#, callback, device_filters)
        dl.monitor.start()

        # BUT BEFORE ALL THAT WE NEED TO ENUMERATE EXISTING DEVICES TO SEE IF IT IS ALREADY THERE

    def notify_connect(self):
        self.statusicon.set_from_stock(Gtk.STOCK_CONNECT)

    def notify_disconnect(self):
        self.statusicon.set_from_stock(Gtk.STOCK_DISCONNECT)

    def notify_charge(self):
        self.statusicon.set_from_stock(Gtk.STOCK_REFRESH)

    def __getattr__(self, name):
        obj = self.builder.get_object(name)
        if obj is None:
            raise AttributeError("Can't resolve '{:s}' from Glade".format(name))
        setattr(self, name, obj)
        return obj

    def on_about_activate(self, widget):
        pass

    def on_quit_activate(self, widget, data = None):
        Gtk.main_quit()

    def on_settings_activate(self, widget):
        # widget.set_sensitive(False)
        self.noshow = True
        try:
            show_settings(self.settings)
        finally:
            self.noshow = False

    def on_statusicon_popup_menu(self, statusicon, button, time):
        def pos(menu, icon):
                return (Gtk.StatusIcon.position_menu(menu, icon))
        if not self.noshow:
            self.menu.popup(None, None, pos, self.statusicon, button, time)

def main():
    i18n.bind()
    t = Schwinn810Tray()
    # print(t.observer.isAlive())
    Gtk.main()
    # print(t.observer.isAlive())

if __name__ == "__main__":
    main()
