import struct, json
from appdirs import AppDirs
from os.path import join
from collections import defaultdict
import schwinn810.constants as config

def unpack_bcd(x0):
    x00 = x0
    x = 0
    m = 0
    while x0:
        digit = x0 & 0x000f
        if digit>9:
            raise Exception("Non-BCD argument {:X} in {:X}".format(digit, x00))
        x = x + digit * 10**m
        m = m + 1
        x0 = x0 >> 4
    return x

def pack_coord(c0, hemi):
    c1 = struct.unpack(">HBHs", c0)
    c2=[unpack_bcd(x) for x in c1[0:3]]
    c = c2[0] + (c2[1]+c2[2]/10000.)/60.
    if (c1[3] == hemi) : c = -c
    return c

def read_settings():
    dirs = AppDirs(config.APP, config.VENDOR)
    fname = join(dirs.user_data_dir, config.FILE)
    try:
        with open(fname, "r") as f:
            settings = json.load(f)
    except IOError:
      # _log.info("Didn't find configuration file %s", fname)
        settings = dict()
#    settings.__missing__ = lambda x: None
    return defaultdict(int, settings)
